/*
 * Copyright (c) 2001-2003 The Trustees of Indiana University.
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame.
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.
 *                         All rights reserved.
 *
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 *
 * $HEADER$
 *
 * $Id: alltoallw.c,v 1.2 2003/10/23 01:09:28 adharurk Exp $
 */

#include "mpi.h"
#include <stdlib.h>
#include <stdio.h>

#include "lamtest_error.h"

/*
  Program to test Alltoallw(), uses only MPI_INT type. Have to update it so 
  that it uses other datatypes as well.
 */

int main( int argc, char **argv )
{

    MPI_Comm comm;
    int      *sbuf, *rbuf;
    int      rank, size;
    MPI_Aint extent;
    int      *sendcounts, *recvcounts, *rdispls, *sdispls;
    int      i, j, *p, err, toterr;
    MPI_Datatype *sdtypes, *rdtypes;
    
    MPI_Init( &argc, &argv );
    err = 0;
    
    comm = MPI_COMM_WORLD;

    /* Create the buffer */
    MPI_Comm_size( comm, &size );
    MPI_Comm_rank( comm, &rank );
    sbuf = (int *)malloc( size * size * sizeof(int) );
    rbuf = (int *)malloc( size * size * sizeof(int) );
    if (!sbuf || !rbuf) {
      lamtest_error(__FILE__, __LINE__, "Doh!  Rank %d was not able to allocateeough memory.  MPI test aborted!\n", rank);
    }
    
    /* Load up the buffers */
    for (i=0; i<size*size; i++) {
	sbuf[i] = i + 100*rank;
	rbuf[i] = -i;
    }

    /* Create and load the arguments to alltoallw */
    sendcounts = (int *)malloc( size * sizeof(int) );
    recvcounts = (int *)malloc( size * sizeof(int) );
    rdispls    = (int *)malloc( size * sizeof(int) );
    sdispls    = (int *)malloc( size * sizeof(int) );
    sdtypes    = (MPI_Datatype *) malloc(size * sizeof(MPI_Datatype));
    rdtypes    = (MPI_Datatype *) malloc(size * sizeof(MPI_Datatype));
    if (!sendcounts || !recvcounts || !rdispls || !sdispls) {
      lamtest_error(__FILE__, __LINE__, "Doh!  Rank %d was not able to allocateenough memory.  MPI test aborted!\n", rank);
    }
    MPI_Type_extent(MPI_INT, &extent);
    for (i=0; i<size; i++) {
	sendcounts[i] = i;
	recvcounts[i] = rank;
	rdispls[i]    = i * rank * extent;
	sdispls[i]    = ((i * (i+1))/2) * extent;
	sdtypes[i] = MPI_INT;
	rdtypes[i] = MPI_INT;
    }
    MPI_Alltoallw( sbuf, sendcounts, sdispls, sdtypes,
		   rbuf, recvcounts, rdispls, rdtypes, comm );

    /* Check rbuf */
    for (i=0; i<size; i++) {
	p = rbuf + (rdispls[i] / extent);
	for (j=0; j<rank; j++) {
	    if (p[j] != i * 100 + (rank*(rank+1))/2 + j) {
	      lamtest_error(__FILE__, __LINE__, "bad answer (%d) (should be %d)\n",p[j], i * 100 + (rank*(rank+1))/2 + j);
	    }
	}
    }

    free( sdispls );
    free( rdispls );
    free( sdtypes );
    free( rdtypes );
    free( recvcounts );
    free( sendcounts );
    free( rbuf );
    free( sbuf );

    MPI_Barrier(MPI_COMM_WORLD);
    MPI_Finalize();
    return 0;
}





