/*
 * Copyright (c) 2001-2004 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 * 
 * $HEADER$
 *
 * $Id: attrsetget.c,v 1.4 2002/10/09 20:55:30 brbarret Exp $
 *
 * This contains the C counterpart of setting and getting attribute
 * from a comm. The functions here will be called from within a
 * fortran program to test language interoperability for attributes
 */

#include <mpi.h>

#include "lamtest_error.h"


/* Prototype */

void attr_set_c(int *key, void *attr);
void attr_get_c(int *key, int *f_attr);
void create_keyval_c(int *key);

/* Actual C set function */

void
attr_set_c(int *key, void *attr)
{
    int ret;
    ret = MPI_Comm_set_attr(MPI_COMM_WORLD, *key, (void *) (*(int *)attr));
    if (ret != MPI_SUCCESS) {
	lamtest_error(__FILE__, __LINE__, "Setting attribute failed from "
		      "within a C function \n");
    }
}


/* Actual C get function */

void
attr_get_c(int *key, int *f_attr)
{
    int *attr;
    char errstr[MPI_MAX_ERROR_STRING];
    int ret, flag = 1;
    int resultlen;
    
    attr = (int *) malloc (sizeof(int));

    MPI_Comm_set_errhandler(MPI_COMM_WORLD, MPI_ERRORS_RETURN);

    ret = MPI_Comm_get_attr(MPI_COMM_WORLD, *key, &attr, &flag);

    MPI_Error_string(ret, errstr, &resultlen);

    if ((ret != MPI_SUCCESS) || (!flag) || (*attr != *f_attr)) {
	lamtest_error(__FILE__, __LINE__, "Getting attribute failed from "
		      "within a C function \n");
    }
}


/* Actual C createvalue function */

void
create_keyval_c(int *key)
{
    void *extra_state;
    int ret;
    ret = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN, 
			   MPI_COMM_NULL_DELETE_FN, key, extra_state);
    if (ret != MPI_SUCCESS) {
	lamtest_error(__FILE__, __LINE__, "Creating keyval failed from "
		      "within a C function \n");
    }
    
}


/* Fortran symbol wrappers */

void attr_set_c_(int *key, void *attr) {
    attr_set_c(key, attr);
}

void attr_set_c__(int *key, void *attr) {
    attr_set_c(key, attr);
}

void ATTR_SET_C(int *key, void *attr) {
    attr_set_c(key, attr);
}



void attr_get_c_(int *key, int *f_attr) {
    attr_get_c(key, f_attr);
}

void attr_get_c__(int *key, int *f_attr) {
    attr_get_c(key, f_attr);
}

void ATTR_GET_C(int *key, int *f_attr) {
    attr_get_c(key, f_attr);
}


void create_keyval_c_(int *key) {
    create_keyval_c(key);
}

void create_keyval_c__(int *key) {
    create_keyval_c(key);
}

void CREATE_KEYVAL_C_(int *key) {
    create_keyval_c(key);
}
