/*
 * Copyright (c) 2001-2003 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 * 
 * $HEADER$
 *
 * $Id: 30_get_cxx.cc,v 1.2 2003/05/04 14:00:17 jsquyres Exp $
 */

#include <mpi.h>

#include "lamtest_error.h"


int
main(int argc, char** argv)
{
  int i, size, rank;
  int *winArea, *rcvArea;
  MPI::Info info;
  MPI::Win win;

  MPI::Init(argc, argv);

  size = MPI::COMM_WORLD.Get_size();
  rank = MPI::COMM_WORLD.Get_rank();

  winArea = (int*) MPI::Alloc_mem(sizeof(int), MPI::INFO_NULL);
  rcvArea = (int*) MPI::Alloc_mem(sizeof(int) * size, MPI::INFO_NULL);
  win = MPI::Win::Create(winArea, sizeof(int), sizeof(int),
                         MPI::INFO_NULL, MPI::COMM_WORLD);

  *winArea = rank;
  win.Fence(0);

  for (i = 0; i < size; ++i) {
    rcvArea[i] = -1;
    win.Get(&rcvArea[i], 1, MPI::INT, i, 0, 1, MPI::INT);
  }
  win.Fence(0);

  for (i = 0; i < size; ++i)
    if (rcvArea[i] != i)
      lamtest_error(__FILE__, __LINE__,
                    "Rank %d got rcvArea[%d]=%d when expecting %d\n",
                    rank, i, rcvArea[i], i);

  win.Free();

  MPI::Free_mem(winArea);
  MPI::Free_mem(rcvArea);
  MPI::Finalize();
  return(0);
}
